@@ -4,7 +4,7 @@
 ****************************************************************************
  install.py, Linux install script for TreeLine
 
- Copyright (C) 2015, Douglas W. Bell
+ Copyright (C) 2018, Douglas W. Bell
 
  This is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License, either Version 2 or any later
@@ -28,7 +28,7 @@ buildRoot = '/'
 progName = 'treeline'
 docDir = 'share/doc/{0}'.format(progName)
 templateDir = 'share/{0}/templates'.format(progName)
-iconDir = 'share/icons/{0}'.format(progName)
+iconToolDir = 'share/icons/{0}'.format(progName)
 testSpell = True
 
 def usage(exitCode=2):
@@ -49,7 +49,8 @@ def usage(exitCode=2):
           .format(docDir))
     print('    -t dir     template dir [default: <prefix>/{0}]'
           .format(templateDir))
-    print('    -i dir     icon dir [default: <prefix>/{0}]'.format(iconDir))
+    print('    -i dir     tool icon dir [default: <prefix>/{0}]'
+          .format(iconToolDir))
     print('    -b dir     temporary build root for packagers [default: {0}]'
           .format(buildRoot))
     print('    -s         skip language translation files')
@@ -187,7 +188,7 @@ def main():
     global prefixDir
     global docDir
     global templateDir
-    global iconDir
+    global iconToolDir
     global buildRoot
     global progName
     depCheck = True
@@ -202,7 +203,7 @@ def main():
         elif opt == '-t':
             templateDir = val
         elif opt == '-i':
-            iconDir = val
+            iconToolDir = val
         elif opt == '-b':
             buildRoot = val
         elif opt == '-s':
@@ -222,31 +223,30 @@ def main():
         print('Checking dependencies...')
         pyVersion = sys.version_info[:3]
         pyVersion = '.'.join([str(num) for num in pyVersion])
-        if cmpVersions(pyVersion, (3, 2)):
+        if cmpVersions(pyVersion, (3, 5)):
             print('  Python Version {0} -> OK'.format(pyVersion))
         else:
-            print('  Python Version {0} -> Sorry, 3.2 or higher is required'
+            print('  Python Version {0} -> Sorry, 3.5 or higher is required'
                   .format(pyVersion))
             sys.exit(3)
         try:
-            from PyQt4 import QtCore, QtGui
+            from PyQt5 import QtCore, QtWidgets
         except:
-            print('  PyQt not found -> Sorry, PyQt 4.x is required'
+            print('  PyQt not found -> Sorry, PyQt 5.8 or higher is required'
                   ' and must be built for Python 3')
             sys.exit(3)
         qtVersion = QtCore.qVersion()
-        if cmpVersions(qtVersion, (4, 6)) and not qtVersion.startswith('5'):
+        if cmpVersions(qtVersion, (5, 8)):
             print('  Qt Version {0} -> OK'.format(qtVersion))
         else:
-            print('  Qt Version {0} -> Sorry, 4.6, 4.7 or 4.8 is required'
+            print('  Qt Version {0} -> Sorry, 5.8 or higher is required'
                   .format(qtVersion))
             sys.exit(3)
         pyqtVersion = QtCore.PYQT_VERSION_STR
-        if (cmpVersions(pyqtVersion, (4, 8)) and
-            not pyqtVersion.startswith('5')):
+        if cmpVersions(pyqtVersion, (5, 8)):
             print('  PyQt Version {0} -> OK'.format(pyqtVersion))
         else:
-            print('  PyQt Version {0} -> Sorry, 4.8 through 4.11 is required'
+            print('  PyQt Version {0} -> Sorry, 5.8 or higher is required'
                   .format(pyqtVersion))
             sys.exit(3)
         global testSpell
@@ -257,15 +257,10 @@ def main():
     pythonBuildDir = os.path.join(buildRoot, pythonPrefixDir[1:])
 
     if os.path.isdir('source'):
-        compileall.compile_dir('source',
-                               ddir=os.path.join(prefixDir, 'source'))
         print('Installing files...')
         print('  Copying python files to {0}'.format(pythonBuildDir))
         removeDir(pythonBuildDir)         # remove old?
         copyDir('source', pythonBuildDir)
-        if os.path.isdir('source/__pycache__'):
-            copyDir('source/__pycache__',
-                    os.path.join(pythonBuildDir, '__pycache__'))
     if os.path.isfile('source/plugininterface.py'):
         pluginBuildDir = os.path.join(pythonBuildDir, 'plugins')
         print('  Creating plugins directory if necessary')
@@ -295,7 +290,7 @@ def main():
             sampleBuildDir = os.path.join(docBuildDir, 'samples')
             print('  Copying sample files to {0}'.format(sampleBuildDir))
             copyDir('samples', sampleBuildDir)
-            # update help file location in main python script
+            # update sample file location in main python script
             replaceLine(os.path.join(pythonBuildDir,
                                      '{0}.py'.format(progName)),
                         'samplePath = None',
@@ -312,11 +307,14 @@ def main():
             for name in glob.glob(os.path.join(templateBuildDir, '*.trl')):
                 if 'en_' not in os.path.basename(name):
                     os.remove(name)
-        # update help file location in main python script
+        # update template file location in main python script
         replaceLine(os.path.join(pythonBuildDir, '{0}.py'.format(progName)),
                     'templatePath = None',
                     'templatePath = \'{0}\'   # modified by install script\n'
                     .format(templatePrefixDir))
+        if os.path.isdir('templates/exports'):
+            exportsBuildDir = os.path.join(templateBuildDir, 'exports')
+            copyDir('templates/exports', exportsBuildDir)
     if os.path.isdir('data'):
         dataPrefixDir = os.path.join(prefixDir, 'share', progName, 'data')
         dataBuildDir = os.path.join(buildRoot, dataPrefixDir[1:])
@@ -333,11 +331,11 @@ def main():
                     'dataFilePath =  \'{0}\'   # modified by install script\n'
                     .format(dataPrefixDir))
     if os.path.isdir('icons'):
-        iconPrefixDir = iconDir.replace('<prefix>/', '')
+        iconPrefixDir = iconToolDir.replace('<prefix>/', '')
         if not os.path.isabs(iconPrefixDir):
             iconPrefixDir = os.path.join(prefixDir, iconPrefixDir)
         iconBuildDir = os.path.join(buildRoot, iconPrefixDir[1:])
-        print('  Copying icon files to {0}'.format(iconBuildDir))
+        print('  Copying tool icon files to {0}'.format(iconBuildDir))
         copyDir('icons', iconBuildDir)
         # update icon location in main python script
         replaceLine(os.path.join(pythonBuildDir, '{0}.py'.format(progName)),
@@ -355,6 +353,31 @@ def main():
                         os.path.join(iconToolBuildDir, '32x32'))
         if os.path.isdir('icons/tree'):
             copyDir('icons/tree', os.path.join(iconBuildDir, 'tree'))
+        if os.path.isfile(os.path.join('icons', progName + '-icon.png')):
+            pngIconPrefixDir = os.path.join(prefixDir, 'share', 'icons',
+                                            'hicolor', '48x48', 'apps')
+            pngIconBuildDir = os.path.join(buildRoot, pngIconPrefixDir[1:])
+            print('  Copying app icon files to {0}'.format(pngIconBuildDir))
+            if not os.path.isdir(pngIconBuildDir):
+                os.makedirs(pngIconBuildDir)
+            shutil.copy2(os.path.join('icons', progName + '-icon.png'),
+                         pngIconBuildDir)
+        if os.path.isfile(os.path.join('icons', progName + '-icon.svg')):
+            svgIconPrefixDir = os.path.join(prefixDir, 'share', 'icons',
+                                            'hicolor', 'scalable', 'apps')
+            svgIconBuildDir = os.path.join(buildRoot, svgIconPrefixDir[1:])
+            print('  Copying app icon files to {0}'.format(svgIconBuildDir))
+            if not os.path.isdir(svgIconBuildDir):
+                os.makedirs(svgIconBuildDir)
+            shutil.copy2(os.path.join('icons', progName + '-icon.svg'),
+                         svgIconBuildDir)
+    if os.path.isfile(progName + '.desktop'):
+        desktopPrefixDir = os.path.join(prefixDir, 'share', 'applications')
+        desktopBuildDir = os.path.join(buildRoot, desktopPrefixDir[1:])
+        print('  Copying desktop file to {0}'.format(desktopBuildDir))
+        if not os.path.isdir(desktopBuildDir):
+            os.makedirs(desktopBuildDir)
+        shutil.copy2(progName + '.desktop', desktopBuildDir)
 
     if os.path.isdir('source'):
         createWrapper(pythonPrefixDir, progName)
@@ -364,6 +387,7 @@ def main():
         if not os.path.isdir(binBuildDir):
             os.makedirs(binBuildDir)
         shutil.copy2(progName, binBuildDir)
+        compileall.compile_dir(pythonBuildDir, ddir=prefixDir)
         cleanSource()
         print('Install complete.')
 
