@@ -4,12 +4,12 @@
 # dataeditview.py, provides a class for the data edit right-hand view
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 from PyQt5.QtCore import QEvent, QPointF, QRectF, QSize, Qt, pyqtSignal
@@ -48,6 +48,8 @@ class DataEditCell(QTableWidgetItem):
         self.titleCellRef = titleCellRef
         self.typeCellRef = typeCellRef
         self.errorFlag = False
+        self.cursorPos = (-1, -1)
+        self.scrollPos = -1
         # store doc to speed up delegate sizeHint and paint calls
         self.doc = QTextDocument()
         self.doc.setDefaultFont(defaultFont)
@@ -71,6 +73,15 @@ class DataEditCell(QTableWidgetItem):
         else:
             self.doc.setPlainText(self.text())
 
+    def storeEditorState(self, editor):
+        """Store the cursor & scroll positions baseed on an editor signal.
+
+        Arguments:
+            editor -- the editor that will get its state saved
+        """
+        self.cursorPos = editor.cursorPosTuple()
+        self.scrollPos = editor.scrollPosition()
+
 
 class DataEditDelegate(QStyledItemDelegate):
     """Class override for display and editing of DataEditCells.
@@ -83,6 +94,7 @@ class DataEditDelegate(QStyledItemDelegate):
         """
         super().__init__(parent)
         self.editorClickPos = None
+        self.tallEditScrollPos = -1
         self.lastEditor = None
         self.prevNumLines = -1
 
@@ -133,7 +145,8 @@ class DataEditDelegate(QStyledItemDelegate):
             doc.setTextWidth(styleOption.rect.width())
             size = doc.documentLayout().documentSize().toSize()
             maxHeight = self.parent().height() * 9 // 10  # 90% of view height
-            if size.height() > maxHeight:
+            if (size.height() > maxHeight and
+                globalref.genOptions['EditorLimitHeight']):
                 size.setHeight(maxHeight)
             if cell.field.numLines > 1:
                 minDoc = QTextDocument('\n' * (cell.field.numLines - 1))
@@ -165,6 +178,10 @@ class DataEditDelegate(QStyledItemDelegate):
                 editor.setErrorFlag()
             # self.parent().setFocusProxy(editor)
             editor.contentsChanged.connect(self.commitData)
+            editor.editEnding.connect(cell.storeEditorState)
+            if (not globalref.genOptions['EditorLimitHeight'] and
+                hasattr(editor, 'keyPressed')):
+                editor.keyPressed.connect(self.scrollOnKeyPress)
             if hasattr(editor, 'inLinkSelectMode'):
                 editor.inLinkSelectMode.connect(self.parent().
                                                 changeInLinkSelectMode)
@@ -205,8 +222,22 @@ class DataEditDelegate(QStyledItemDelegate):
             if self.editorClickPos:
                 editor.setCursorPoint(self.editorClickPos)
                 self.editorClickPos = None
-            else:
-                editor.resetCursor()
+            elif globalref.genOptions['EditorLimitHeight']:
+                if cell.cursorPos[1] >= 0:
+                    editor.setCursorPos(*cell.cursorPos)
+                    cell.cursorPos = (-1, -1)
+                    if cell.scrollPos >= 0:
+                        editor.setScrollPosition(cell.scrollPos)
+                        cell.scrollPos = -1
+                else:
+                    editor.resetCursor()
+            if (not globalref.genOptions['EditorLimitHeight'] and
+                globalref.genOptions['EditorOnHover'] and
+                self.tallEditScrollPos >= 0):
+                # maintain scroll position for unlimited height editors
+                # when hovering (use adjustScroll() for non-hovering
+                self.parent().verticalScrollBar().setValue(self.
+                                                           tallEditScrollPos)
         else:
             super().setEditorData(editor, modelIndex)
 
@@ -251,6 +282,39 @@ class DataEditDelegate(QStyledItemDelegate):
         editor.setMaximumSize(self.sizeHint(styleOption, modelIndex))
         super().updateEditorGeometry(editor, styleOption, modelIndex)
 
+    def adjustScroll(self):
+        """Reset the scroll back to the original for unlimited height editors.
+
+        Called from signal after any scroll change.
+        Needed for non-hovering to fix late scroll reset after editor created.
+        """
+        if (not globalref.genOptions['EditorLimitHeight'] and
+            not globalref.genOptions['EditorOnHover'] and
+            self.tallEditScrollPos >= 0):
+            self.parent().verticalScrollBar().setValue(self.tallEditScrollPos)
+            self.tallEditScrollPos = -1
+
+    def scrollOnKeyPress(self, editor):
+        """Adjust the scroll position to make cursor visible.
+
+        Needed after key presses on unlimited height editors.
+        Arguments:
+            editor -- the editor with the key press
+        """
+        if not globalref.genOptions['EditorLimitHeight']:
+            view = self.parent()
+            cursorRect = editor.cursorRect()
+            upperPos = editor.mapToGlobal(cursorRect.topLeft()).y()
+            lowerPos = editor.mapToGlobal(cursorRect.bottomLeft()).y()
+            viewRect = view.viewport().rect()
+            viewTop = view.mapToGlobal(viewRect.topLeft()).y()
+            viewBottom = view.mapToGlobal(viewRect.bottomLeft()).y()
+            bar = view.verticalScrollBar()
+            if upperPos < viewTop:
+                bar.setValue(bar.value() - (viewTop - upperPos))
+            elif lowerPos > viewBottom:
+                bar.setValue(bar.value() + (lowerPos - viewBottom))
+
     def editorEvent(self, event, model, styleOption, modelIndex):
         """Save the mouse click position in order to set the editor's cursor.
 
@@ -262,6 +326,8 @@ class DataEditDelegate(QStyledItemDelegate):
         """
         if event.type() == QEvent.MouseButtonPress:
             self.editorClickPos = event.globalPos()
+            # save scroll position for clicks on unlimited height editors
+            self.tallEditScrollPos = self.parent().verticalScrollBar().value()
         return super().editorEvent(event, model, styleOption, modelIndex)
 
     def eventFilter(self, editor, event):
@@ -338,6 +404,8 @@ class DataEditView(QTableWidget):
         self.verticalHeader().hide()
         self.setVerticalScrollMode(QAbstractItemView.ScrollPerPixel)
         self.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
+        self.verticalScrollBar().setSingleStep(self.fontMetrics().
+                                               lineSpacing())
         self.setSelectionMode(QAbstractItemView.SingleSelection)
         self.setItemDelegate(DataEditDelegate(self))
         self.setEditTriggers(QAbstractItemView.NoEditTriggers)
@@ -349,6 +417,8 @@ class DataEditView(QTableWidget):
                      QApplication.palette().windowText())
         self.setPalette(pal)
         self.currentItemChanged.connect(self.moveEditor)
+        self.verticalScrollBar().valueChanged.connect(self.itemDelegate().
+                                                      adjustScroll)
 
     def updateContents(self):
         """Reload the view's content if the view is shown.
@@ -406,6 +476,7 @@ class DataEditView(QTableWidget):
         if not globalref.genOptions['ShowMath']:
             fields = [field for field in fields
                       if field.typeName != 'Math']
+        row = 0  # initialize for cases with only Numbering or Math fields
         for row, field in enumerate(fields, startRow + 1):
             self.setItem(row, 0, self.createInactiveCell(field.name,
                                                          Qt.AlignRight |
@@ -478,10 +549,13 @@ class DataEditView(QTableWidget):
             newCell -- the new current edit cell item
             prevCell - the old current cell item
         """
-        if prevCell and hasattr(prevCell, 'updateText'):
-            self.closePersistentEditor(prevCell)
-            prevCell.updateText()
-            self.resizeRowToContents(prevCell.row())
+        try:
+            if prevCell and hasattr(prevCell, 'updateText'):
+                self.closePersistentEditor(prevCell)
+                prevCell.updateText()
+                self.resizeRowToContents(prevCell.row())
+        except RuntimeError:
+            pass   # avoid non-repeatable error involving deleted c++ object
         if newCell:
             self.openPersistentEditor(newCell)
 
@@ -697,6 +771,10 @@ class DataEditView(QTableWidget):
             oldCell = self.currentItem()
             if (cell != oldCell and cell != self.prevHoverCell and
                 not self.inLinkSelectActive):
+                # save scroll position for unlimited height editors
+                self.itemDelegate().tallEditScrollPos = (self.
+                                                         verticalScrollBar().
+                                                         value())
                 self.prevHoverCell = cell
                 self.hoverFocusActive.emit()
                 self.setFocus()
@@ -756,8 +834,8 @@ class MatchHighlighter(QSyntaxHighlighter):
         Arguments:
             doc -- the text document
             charFormat -- the formatting to apply
-            searchText -- the text to find in a non-regexp search
-            regExpObj -- the regular expression to find if searchText is blank
+            searchText -- the text to find if no regexp is given
+            regExpObj -- the regular expression to find if given
             skipMatches -- number of previous matches to skip
         """
         super().__init__(doc)
@@ -775,12 +853,12 @@ class MatchHighlighter(QSyntaxHighlighter):
         pos = matchLen = 0
         for matchNum in range(self.skipMatches + 1):
             pos += matchLen
-            if self.searchText:
-                pos = text.lower().find(self.searchText, pos)
-                matchLen = len(self.searchText)
-            else:
+            if self.regExpObj:
                 match = self.regExpObj.search(text, pos)
                 pos = match.start() if match else -1
                 matchLen = len(match.group()) if match else 0
+            else:
+                pos = text.lower().find(self.searchText, pos)
+                matchLen = len(self.searchText)
         if pos >= 0:
             self.setFormat(pos, matchLen, self.charFormat)
