@@ -4,12 +4,12 @@
 # fieldformat.py, provides a class to handle field format types
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import re
@@ -39,6 +39,8 @@ _mathResultBlank = {MathResult.number: 0, MathResult.date: 0,
                     MathResult.time: 0, MathResult.boolean: False,
                     MathResult.text: ''}
 _multipleSpaceRegEx = re.compile(r' {2,}')
+_lineBreakRegEx = re.compile(r'<br\s*/?>', re.I)
+_stripTagRe = re.compile(r'<.*?>')
 linkRegExp = re.compile(r'<a [^>]*href="([^"]+)"[^>]*>(.*?)</a>', re.I | re.S)
 linkSeparateNameRegExp = re.compile(r'(.*) \[(.*)\]\s*$')
 _imageRegExp = re.compile(r'<img [^>]*src="([^"]+)"[^>]*>', re.I | re.S)
@@ -115,12 +117,13 @@ class TextField:
         """
         self.format = format
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Arguments:
             node -- the tree item storing the data
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
@@ -129,25 +132,28 @@ class TextField:
             node = node.treeStructureRef().fileInfoNode
         storedText = node.data.get(self.name, '')
         if storedText:
-            return self.formatOutput(storedText, titleMode, formatHtml)
+            return self.formatOutput(storedText, oneLine, noHtml, formatHtml)
         return ''
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         prefix = self.prefix
         suffix = self.suffix
-        if titleMode:
+        if oneLine:
+            storedText = _lineBreakRegEx.split(storedText, 1)[0]
+        if noHtml:
             storedText = removeMarkup(storedText)
             if formatHtml:
                 prefix = removeMarkup(prefix)
                 suffix = removeMarkup(suffix)
-        elif not formatHtml:
+        if not formatHtml:
             prefix = saxutils.escape(prefix)
             suffix = saxutils.escape(suffix)
         return '{0}{1}{2}'.format(prefix, storedText, suffix)
@@ -222,16 +228,18 @@ class TextField:
         """
         return []
 
-    def mathValue(self, node, zeroBlanks=True):
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a value to be used in math field equations.
 
         Return None if blank and not zeroBlanks.
         Arguments:
             node -- the tree item storing the data
             zeroBlanks -- accept blank field values if True
+            noMarkup -- if true, remove html markup
         """
         storedText = node.data.get(self.name, '')
-        storedText = removeMarkup(storedText)
+        if storedText and noMarkup:
+            storedText = removeMarkup(storedText)
         return storedText if storedText or zeroBlanks else None
 
     def compareValue(self, node):
@@ -334,16 +342,17 @@ class OneLineTextField(TextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
-        text = storedText.split('<br />', 1)[0]
-        return super().formatOutput(text, titleMode, formatHtml)
+        text = _lineBreakRegEx.split(storedText, 1)[0]
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -352,7 +361,7 @@ class OneLineTextField(TextField):
         Arguments:
             storedText -- the source text to format
         """
-        return storedText.split('<br />', 1)[0]
+        return _lineBreakRegEx.split(storedText, 1)[0]
 
 
 class SpacedTextField(TextField):
@@ -375,17 +384,18 @@ class SpacedTextField(TextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         if storedText:
             storedText = '<pre>{0}</pre>'.format(storedText)
-        return super().formatOutput(storedText, titleMode, formatHtml)
+        return super().formatOutput(storedText, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -449,19 +459,20 @@ class NumberField(HtmlTextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         try:
             text = gennumber.GenNumber(storedText).numStr(self.format)
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -485,7 +496,7 @@ class NumberField(HtmlTextField):
             return ''
         return repr(gennumber.GenNumber().setFromStr(editorText, self.format))
 
-    def mathValue(self, node, zeroBlanks=True):
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a numeric value to be used in math field equations.
 
         Return None if blank and not zeroBlanks,
@@ -493,6 +504,7 @@ class NumberField(HtmlTextField):
         Arguments:
             node -- the tree item storing the data
             zeroBlanks -- replace blank field values with zeros if True
+            noMarkup -- not applicable to numbers
         """
         storedText = node.data.get(self.name, '')
         if storedText:
@@ -521,6 +533,7 @@ class MathField(HtmlTextField):
     typeName = 'Math'
     defaultFormat = '#.##'
     evalHtmlDefault = False
+    fixEvalHtmlSetting = False
     editorClassName = 'ReadOnlyEditor'
     def __init__(self, name, formatData=None):
         """Initialize a field format type.
@@ -563,12 +576,13 @@ class MathField(HtmlTextField):
             self.resultType = MathResult.number
         super().setFormat(format)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         text = storedText
@@ -587,7 +601,7 @@ class MathField(HtmlTextField):
                 text =  genboolean.GenBoolean(text).boolStr(self.format)
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -633,9 +647,10 @@ class MathField(HtmlTextField):
             node -- the tree item with this equation
         """
         if self.equation:
+            zeroValue = _mathResultBlank[self.resultType]
             try:
-                num = self.equation.equationValue(node,
-                                             _mathResultBlank[self.resultType])
+                num = self.equation.equationValue(node, self.resultType,
+                                                  zeroValue, not self.evalHtml)
             except ValueError:
                 return _errorStr
             if num == None:
@@ -649,7 +664,10 @@ class MathField(HtmlTextField):
                 dateTime = dateTime + datetime.timedelta(seconds=num)
                 time = dateTime.time()
                 return time.strftime(TimeField.isoFormat)
-            return str(num)
+            text = str(num)
+            if not self.evalHtml:
+                text = saxutils.escape(text)
+            return text
         return ''
 
     def resultClass(self):
@@ -667,7 +685,7 @@ class MathField(HtmlTextField):
             self.resultType = resultType
             self.setFormat(self.resultClass().defaultFormat)
 
-    def mathValue(self, node, zeroBlanks=True):
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a numeric value to be used in math field equations.
 
         Return None if blank and not zeroBlanks,
@@ -675,6 +693,7 @@ class MathField(HtmlTextField):
         Arguments:
             node -- the tree item storing the data
             zeroBlanks -- replace blank field values with zeros if True
+            noMarkup -- if true, remove html markup
         """
         storedText = node.data.get(self.name, '')
         if storedText:
@@ -690,7 +709,9 @@ class MathField(HtmlTextField):
                 return (time - TimeField.refTime).seconds
             if self.resultType == MathResult.boolean:
                 return  genboolean.GenBoolean(storedText).value
-            return removeMarkup(storedText)
+            if noMarkup:
+                storedText = removeMarkup(storedText)
+            return storedText
         return _mathResultBlank[self.resultType] if zeroBlanks else None
 
     def adjustedCompareValue(self, value):
@@ -778,19 +799,20 @@ class NumberingField(HtmlTextField):
         self.numFormat = numbering.NumberingGroup(format)
         super().setFormat(format)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         try:
             text = self.numFormat.numString(storedText)
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -839,6 +861,7 @@ class DateField(HtmlTextField):
     defaultFormat = '%B %-d, %Y'
     isoFormat = '%Y-%m-%d'
     evalHtmlDefault = False
+    fixEvalHtmlSetting = False
     editorClassName = 'DateEditor'
     refDate = datetime.date(1970, 1, 1)
     sortTypeStr = '40_date'
@@ -863,12 +886,13 @@ class DateField(HtmlTextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         try:
@@ -877,7 +901,9 @@ class DateField(HtmlTextField):
             text = date.strftime(adjOutDateFormat(self.format))
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        if not self.evalHtml:
+            text = saxutils.escape(text)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -948,7 +974,7 @@ class DateField(HtmlTextField):
         """
         return [_dateStampString]
 
-    def mathValue(self, node, zeroBlanks=True):
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a numeric value to be used in math field equations.
 
         Return None if blank and not zeroBlanks,
@@ -1003,6 +1029,7 @@ class TimeField(HtmlTextField):
     defaultFormat = '%-I:%M:%S %p'
     isoFormat = '%H:%M:%S.%f'
     evalHtmlDefault = False
+    fixEvalHtmlSetting = False
     editorClassName = 'TimeEditor'
     numChoiceColumns = 2
     autoAddChoices = False
@@ -1027,12 +1054,13 @@ class TimeField(HtmlTextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         try:
@@ -1043,7 +1071,9 @@ class TimeField(HtmlTextField):
             text = time.strftime(outFormat)
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        if not self.evalHtml:
+            text = saxutils.escape(text)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -1140,7 +1170,7 @@ class TimeField(HtmlTextField):
         """
         return [_timeStampString]
 
-    def mathValue(self, node, zeroBlanks=True):
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a numeric value to be used in math field equations.
 
         Return None if blank and not zeroBlanks,
@@ -1198,6 +1228,7 @@ class DateTimeField(HtmlTextField):
     defaultFormat = '%B %-d, %Y %-I:%M:%S %p'
     isoFormat = '%Y-%m-%d %H:%M:%S.%f'
     evalHtmlDefault = False
+    fixEvalHtmlSetting = False
     editorClassName = 'DateTimeEditor'
     refDateTime = datetime.datetime(1970, 1, 1)
     sortTypeStr ='45_datetime'
@@ -1232,12 +1263,13 @@ class DateTimeField(HtmlTextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         try:
@@ -1248,7 +1280,9 @@ class DateTimeField(HtmlTextField):
             text = dateTime.strftime(outFormat)
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        if not self.evalHtml:
+            text = saxutils.escape(text)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -1338,7 +1372,7 @@ class DateTimeField(HtmlTextField):
         """
         return [_timeStampString]
 
-    def mathValue(self, node, zeroBlanks=True):
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a numeric value to be used in math field equations.
 
         Return None if blank and not zeroBlanks,
@@ -1351,7 +1385,7 @@ class DateTimeField(HtmlTextField):
         if storedText:
             dateTime = datetime.datetime.strptime(storedText,
                                                   DateTimeField.isoFormat)
-            return (dateTime - DateTimeField.refDateTime).seconds
+            return (dateTime - DateTimeField.refDateTime).total_seconds()
         return 0 if zeroBlanks else None
 
     def compareValue(self, node):
@@ -1423,17 +1457,18 @@ class ChoiceField(HtmlTextField):
             self.choices = set([saxutils.escape(choice) for choice in
                                 self.choiceList])
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         if storedText not in self.choices:
             storedText = _errorStr
-        return super().formatOutput(storedText, titleMode, formatHtml)
+        return super().formatOutput(storedText, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -1586,25 +1621,27 @@ class CombinationField(ChoiceField):
         self.choices = set(self.choiceList)
         self.outputSep = ''
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Sets output separator prior to calling base class methods.
         Arguments:
             node -- the tree item storing the data
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
         self.outputSep = node.formatRef.outputSeparator
-        return super().outputText(node, titleMode, formatHtml, spotRef)
+        return super().outputText(node, oneLine, noHtml, formatHtml, spotRef)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         selections, valid = self.sortedSelections(storedText)
@@ -1612,7 +1649,8 @@ class CombinationField(ChoiceField):
             result = self.outputSep.join(selections)
         else:
             result = _errorStr
-        return TextField.formatOutput(self, result, titleMode, formatHtml)
+        return TextField.formatOutput(self, result, oneLine, noHtml,
+                                      formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -1709,29 +1747,32 @@ class AutoCombinationField(CombinationField):
         self.choices = set()
         self.outputSep = ''
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Sets output separator prior to calling base class methods.
         Arguments:
             node -- the tree item storing the data
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
         self.outputSep = node.formatRef.outputSeparator
-        return super().outputText(node, titleMode, formatHtml, spotRef)
+        return super().outputText(node, oneLine, noHtml, formatHtml, spotRef)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         result = self.outputSep.join(self.splitText(storedText))
-        return TextField.formatOutput(self, result, titleMode, formatHtml)
+        return TextField.formatOutput(self, result, oneLine, noHtml,
+                                      formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -1812,7 +1853,7 @@ class BooleanField(ChoiceField):
     typeName = 'Boolean'
     defaultFormat = _('yes/no')
     evalHtmlDefault = False
-    fixEvalHtmlSetting = True
+    fixEvalHtmlSetting = False
     sortTypeStr ='30_bool'
     formatHelpMenuList = [(_('true/false'), 'true/false'),
                           (_('T/F'), 'T/F'), ('', ''),
@@ -1828,19 +1869,32 @@ class BooleanField(ChoiceField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def setFormat(self, format):
+        """Set the format string and initialize as required.
+
+        Arguments:
+            format -- the new format string
+        """
+        HtmlTextField.setFormat(self, format)
+        self.strippedFormat = removeMarkup(self.format)
+
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         try:
             text =  genboolean.GenBoolean(storedText).boolStr(self.format)
         except ValueError:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        if not self.evalHtml:
+            text = saxutils.escape(text)
+        return HtmlTextField.formatOutput(self, text, oneLine, noHtml,
+                                          formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -1851,7 +1905,8 @@ class BooleanField(ChoiceField):
         """
         if not storedText:
             return ''
-        return genboolean.GenBoolean(storedText).boolStr(self.format)
+        boolFormat = self.strippedFormat if self.evalHtml else self.format
+        return genboolean.GenBoolean(storedText).boolStr(boolFormat)
 
     def storedText(self, editorText):
         """Return new text to be stored based on text from the data editor.
@@ -1862,13 +1917,26 @@ class BooleanField(ChoiceField):
         """
         if not editorText:
             return ''
+        boolFormat = self.strippedFormat if self.evalHtml else self.format
         try:
             return repr(genboolean.GenBoolean().setFromStr(editorText,
-                                                           self.format))
+                                                           boolFormat))
         except ValueError:
             return repr(genboolean.GenBoolean(editorText))
 
-    def mathValue(self, node, zeroBlanks=True):
+    def comboChoices(self):
+        """Return a list of choices for the combo box.
+        """
+        if self.evalHtml:
+            return self.splitText(self.strippedFormat)
+        return self.splitText(self.format)
+
+    def initDefaultChoices(self):
+        """Return a list of choices for setting the init default.
+        """
+        return self.comboChoices()
+
+    def mathValue(self, node, zeroBlanks=True, noMarkup=True):
         """Return a value to be used in math field equations.
 
         Return None if blank and not zeroBlanks,
@@ -1947,22 +2015,23 @@ class ExternalLinkField(HtmlTextField):
         address, name = linkMatch.groups()
         return (address, name)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
-        if titleMode:
+        if noHtml:
             linkMatch = linkRegExp.search(storedText)
             if linkMatch:
                 address, name = linkMatch.groups()
                 storedText = name.strip()
                 if not storedText:
                     storedText = address.lstrip('#')
-        return super().formatOutput(storedText, titleMode, formatHtml)
+        return super().formatOutput(storedText, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -2109,20 +2178,21 @@ class PictureField(HtmlTextField):
         """
         super().__init__(name, formatData)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
-        if titleMode:
+        if noHtml:
             linkMatch = _imageRegExp.search(storedText)
             if linkMatch:
                 address = linkMatch.group(1)
                 storedText = address.strip()
-        return super().formatOutput(storedText, titleMode, formatHtml)
+        return super().formatOutput(storedText, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -2219,12 +2289,13 @@ class RegularExpressionField(HtmlTextField):
             raise ValueError
         super().setFormat(format)
 
-    def formatOutput(self, storedText, titleMode, formatHtml):
+    def formatOutput(self, storedText, oneLine, noHtml, formatHtml):
         """Return formatted output text from stored text for this field.
 
         Arguments:
             storedText -- the source text to format
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
         """
         match = re.fullmatch(self.format, saxutils.unescape(storedText))
@@ -2232,7 +2303,7 @@ class RegularExpressionField(HtmlTextField):
             text = storedText
         else:
             text = _errorStr
-        return super().formatOutput(text, titleMode, formatHtml)
+        return super().formatOutput(text, oneLine, noHtml, formatHtml)
 
     def formatEditorText(self, storedText):
         """Return text formatted for use in the data editor.
@@ -2277,13 +2348,14 @@ class AncestorLevelField(TextField):
         super().__init__(name, {})
         self.ancestorLevel = ancestorLevel
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Finds the appropriate ancestor node to get the field text.
         Arguments:
             node -- the tree node to start from
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
@@ -2297,7 +2369,7 @@ class AncestorLevelField(TextField):
             field = spotRef.nodeRef.formatRef.fieldDict[self.name]
         except (AttributeError, KeyError):
             return ''
-        return field.outputText(spotRef.nodeRef, titleMode, formatHtml,
+        return field.outputText(spotRef.nodeRef, oneLine, noHtml, formatHtml,
                                 spotRef)
 
     def sepName(self):
@@ -2318,13 +2390,14 @@ class AnyAncestorField(TextField):
         """
         super().__init__(name, {})
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Finds the appropriate ancestor node to get the field text.
         Arguments:
             node -- the tree node to start from
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
@@ -2337,8 +2410,8 @@ class AnyAncestorField(TextField):
             except (AttributeError, KeyError):
                 pass
             else:
-                return field.outputText(spotRef.nodeRef, titleMode, formatHtml,
-                                        spotRef)
+                return field.outputText(spotRef.nodeRef, oneLine, noHtml,
+                                        formatHtml, spotRef)
         return ''
 
     def sepName(self):
@@ -2359,13 +2432,14 @@ class ChildListField(TextField):
         """
         super().__init__(name, {})
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Returns a joined list of matching child field data.
         Arguments:
             node -- the tree node to start from
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
@@ -2376,8 +2450,8 @@ class ChildListField(TextField):
             except KeyError:
                 pass
             else:
-                result.append(field.outputText(child, titleMode, formatHtml,
-                                               spotRef))
+                result.append(field.outputText(child, oneLine, noHtml,
+                                               formatHtml, spotRef))
         outputSep = node.formatRef.outputSeparator
         return outputSep.join(result)
 
@@ -2401,13 +2475,14 @@ class DescendantCountField(TextField):
         super().__init__(name, {})
         self.descendantLevel = descendantLevel
 
-    def outputText(self, node, titleMode, formatHtml, spotRef=None):
+    def outputText(self, node, oneLine, noHtml, formatHtml, spotRef=None):
         """Return formatted output text for this field in this node.
 
         Returns a count of descendants at the approriate level.
         Arguments:
             node -- the tree node to start from
-            titleMode -- if True, removes all HTML markup for tree title use
+            oneLine -- if True, returns only first line of output (for titles)
+            noHtml -- if True, removes all HTML markup (for titles, etc.)
             formatHtml -- if False, escapes HTML from prefix & suffix
             spotRef -- optional, used for ancestor field refs
         """
@@ -2427,11 +2502,12 @@ class DescendantCountField(TextField):
 
 ####  Utility Functions  ####
 
-_stripTagRe = re.compile('<.*?>')
-
 def removeMarkup(text):
     """Return text with all HTML Markup removed and entities unescaped.
+
+    Any <br /> tags are replaced with newlines.
     """
+    text = _lineBreakRegEx.sub('\n', text)
     text = _stripTagRe.sub('', text)
     return saxutils.unescape(text)
 
@@ -2471,4 +2547,9 @@ def translatedTypeName(typeName):
     Arguments:
         typeName -- the English type name
     """
-    return translatedFieldTypes[fieldTypes.index(typeName)]
+    try:
+        return translatedFieldTypes[fieldTypes.index(typeName)]
+    except ValueError:
+        if typeName == 'DescendantCount':
+            return _('DescendantCount')
+        raise
