@@ -4,16 +4,16 @@
 # recentfiles.py, classes to save recent file lists, states and actions
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import pathlib
-import os
+import os.path
 import time
 from PyQt5.QtWidgets import QAction
 import globalref
@@ -135,7 +135,9 @@ class RecentFileItem:
             otherPath = other.pathObj
         except AttributeError:
             otherPath = other
-        return self.pathObj.resolve() == otherPath.resolve()
+        # use abspath() - pathlib's resolve() can be buggy with network drives
+        return (os.path.abspath(str(self.pathObj)) ==
+                os.path.abspath(str(otherPath)))
 
     def __ne__(self, other):
         """Test for inequality between RecentFileItems and paths.
@@ -147,7 +149,9 @@ class RecentFileItem:
             otherPath = other.pathObj
         except AttributeError:
             otherPath = other
-        return self.pathObj.resolve() != otherPath.resolve()
+        # use abspath() - pathlib's resolve() can be buggy with network drives
+        return (os.path.abspath(str(self.pathObj)) !=
+                os.path.abspath(str(otherPath)))
 
 
 class RecentFileList(list):
@@ -247,6 +251,6 @@ class RecentFileList(list):
         """
         try:
             item = self[self.index(localControl.filePathObj)]
-        except ValueError:
+        except (ValueError, AttributeError):
             return False
         return item.restoreTreeState(localControl)
