@@ -4,17 +4,17 @@
 # titlelistview.py, provides a class for the title list view
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2019, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 from PyQt5.QtCore import Qt, pyqtSignal
 from PyQt5.QtGui import QKeySequence, QPalette, QTextCursor
-from PyQt5.QtWidgets import QTextEdit
+from PyQt5.QtWidgets import QAction, QTextEdit
 import treenode
 import undo
 import globalref
@@ -44,6 +44,8 @@ class TitleListView(QTextEdit):
         self.setLineWrapMode(QTextEdit.NoWrap)
         self.setTabChangesFocus(True)
         self.setUndoRedoEnabled(False)
+        self.treeSelectAction = QAction(_('Select in Tree'), self)
+        self.treeSelectAction.triggered.connect(self.selectLineInTree)
         self.textChanged.connect(self.readChange)
 
     def updateContents(self):
@@ -126,6 +128,23 @@ class TitleListView(QTextEdit):
         else:
             self.updateContents()  # remove illegal changes
 
+    def selectLineInTree(self):
+        """Select the node for the current line in the tree view.
+        """
+        selSpots = self.treeView.selectionModel().selectedSpots()
+        if not selSpots:
+            selSpots = [treeStructure.structSpot()]
+        spotList = selSpots[0].childSpots()
+        cursor = self.textCursor()
+        blockStart = cursor.block().position()
+        # check for selection all on one line
+        if (cursor.selectionStart() >= blockStart and
+            cursor.selectionEnd() < blockStart + cursor.block().length()):
+            lineNum = cursor.blockNumber()
+            if len(spotList) > lineNum:
+                self.treeView.selectionModel().selectSpots([spotList[lineNum]],
+                                                           True, True)
+
     def hasSelectedText(self):
         """Return True if text is selected.
         """
@@ -183,6 +202,10 @@ class TitleListView(QTextEdit):
         menu = self.createStandardContextMenu()
         menu.removeAction(menu.actions()[0])
         menu.removeAction(menu.actions()[0])
+        menu.insertSeparator(menu.actions()[0])
+        menu.insertAction(menu.actions()[0], self.treeSelectAction)
+        self.treeSelectAction.setEnabled(self.isChildView and
+                                         len(self.toPlainText().strip()) > 0)
         menu.exec_(event.globalPos())
 
     def keyPressEvent(self, event):
