@@ -4,7 +4,7 @@
 # treelocalcontrol.py, provides a class for the main tree commands
 #
 # TreeLine, an information storage program
-# Copyright (C) 2020, Douglas W. Bell
+# Copyright (C) 2022, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -693,6 +693,12 @@ class TreeLocalControl(QObject):
         formatUnderlineAct.setEnabled(False)
         localActions['FormatUnderlineFont'] = formatUnderlineAct
 
+        formatStrikethroughAct = QAction(_('St&rikethrough Font'), self,
+                statusTip=_('Set the current or selected font to strikethough'),
+                checkable=True)
+        formatStrikethroughAct.setEnabled(False)
+        localActions['FormatStrikethroughFont'] = formatStrikethroughAct
+
         title = _('&Font Size')
         key = globalref.keyboardOptions['FormatFontSize']
         if not key.isEmpty():
