@@ -4,12 +4,12 @@
 # treenode.py, provides a class to store tree node data
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2020, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import re
@@ -339,8 +339,8 @@ class TreeNode:
         found), the node match number and the field match number.
         Returns the last match if skipMatches < 0 (not used with replace).
         Arguments:
-            searchText -- the text to find in a non-regexp search
-            regExpObj -- the regular expression to find if searchText is blank
+            searchText -- the text to find if regExpObj is None
+            regExpObj -- the regular expression to find if not None
             skipMatches -- number of already found matches to skip in this node
             typeName -- if given, verify that this node matches this type
             fieldName -- if given, only find matches under this type name
@@ -349,35 +349,43 @@ class TreeNode:
         """
         if typeName and typeName != self.formatRef.name:
             return ('', 0, 0)
-        fields = ([self.formatRef.fieldDict[fieldName]] if fieldName
-                  else self.formatRef.fields())
+        try:
+            fields = ([self.formatRef.fieldDict[fieldName]] if fieldName
+                      else self.formatRef.fields())
+        except KeyError:
+            return ('', 0, 0)   # field not in this type
         matchedFieldname = ''
         findCount = 0
         prevFieldFindCount = 0
         for field in fields:
-            fieldText = field.editorText(self)
+            try:
+                fieldText = field.editorText(self)
+            except ValueError:
+                fieldText = self.data.get(field.name, '')
             fieldFindCount = 0
             pos = 0
             while True:
-                if pos >= len(fieldText):
+                if pos >= len(fieldText) and pos > 0:
                     break
-                if searchText:
-                    pos = fieldText.lower().find(searchText, pos)
-                else:
+                if regExpObj:
                     match = regExpObj.search(fieldText, pos)
                     pos = match.start() if match else -1
+                else:
+                    pos = fieldText.lower().find(searchText, pos)
+                    if not searchText and fieldText:
+                        pos = -1  # skip invalid find of empty string
                 if pos < 0:
                     break
                 findCount += 1
                 fieldFindCount += 1
                 prevFieldFindCount = fieldFindCount
-                matchLen = (len(searchText) if searchText
-                            else len(match.group()))
+                matchLen = (len(match.group()) if regExpObj
+                            else len(searchText))
                 if findCount > skipMatches:
                     matchedFieldname = field.name
                     if replaceText is not None:
                         replace = replaceText
-                        if not searchText:
+                        if regExpObj:
                             global _origBackrefMatch
                             _origBackrefMatch = match
                             for backrefRe in _replaceBackrefRe:
@@ -391,7 +399,7 @@ class TreeNode:
                             pass
                     if not replaceAll and skipMatches >= 0:
                         return (field.name, findCount, fieldFindCount)
-                pos += matchLen
+                pos = pos + matchLen if matchLen else pos + 1
         if not matchedFieldname:
             findCount = prevFieldFindCount = 0
         return (matchedFieldname, findCount, prevFieldFindCount)
@@ -657,6 +665,9 @@ class TreeNode:
                                                  currentSequence))
             if self.formatRef.name in fieldDict or reserveNums:
                 childSequence += [1]
+                currentSequence[-1] += 1
+            if restartSetting and self.formatRef.name not in fieldDict:
+                currentSequence[-1] = 1
             if len(self.spotRefs) > 1:
                 completedClones.add(self.uId)
         if levelLimit > 0:
@@ -666,10 +677,6 @@ class TreeNode:
                 child.updateNumbering(fieldDict, childSequence, levelLimit - 1,
                                       completedClones, True, reserveNums,
                                       restartSetting)
-                if child.formatRef.name in fieldDict or reserveNums:
-                    childSequence[-1] += 1
-                if restartSetting and child.formatRef.name not in fieldDict:
-                    childSequence[-1] = 1
 
     def isIdentical(self, node, checkParents=True):
         """Return True if node format, data and descendants are identical.
