@@ -4,12 +4,12 @@
 # treewindow.py, provides a class for the main window and controls
 #
 # TreeLine, an information storage program
-# Copyright (C) 2018, Douglas W. Bell
+# Copyright (C) 2022, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
 # version.  This program is distributed in the hope that it will be useful,
-# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
+# but WITHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #******************************************************************************
 
 import pathlib
@@ -36,6 +36,7 @@ class TreeWindow(QMainWindow):
     nodeModified = pyqtSignal(treenode.TreeNode)
     treeModified = pyqtSignal()
     winActivated = pyqtSignal(QMainWindow)
+    winMinimized = pyqtSignal()
     winClosing = pyqtSignal(QMainWindow)
     def __init__(self, model, allActions, parent=None):
         """Initialize the main window.
@@ -264,15 +265,17 @@ class TreeWindow(QMainWindow):
         self.activateWindow()
         self.raise_()
 
-    def setCaption(self, pathObj=None):
+    def setCaption(self, pathObj=None, modified=False):
         """Change the window caption title based on the file name and path.
 
         Arguments:
             pathObj - a path object for the current file
         """
+        modFlag = '*' if modified else ''
         if pathObj:
-            caption = '{0} [{1}] - TreeLine'.format(str(pathObj.name),
-                                                    str(pathObj.parent))
+            caption = '{0}{1} [{2}] - TreeLine'.format(str(pathObj.name),
+                                                       modFlag,
+                                                       str(pathObj.parent))
         else:
             caption = '- TreeLine'
         self.setWindowTitle(caption)
@@ -552,9 +555,11 @@ class TreeWindow(QMainWindow):
         self.addAction(self.allActions['DataNodeType'])
         dataMenu.addAction(self.allActions['DataConfigType'])
         dataMenu.addAction(self.allActions['DataCopyType'])
+        dataMenu.addAction(self.allActions['DataVisualConfig'])
         dataMenu.addSeparator()
         dataMenu.addAction(self.allActions['DataSortNodes'])
         dataMenu.addAction(self.allActions['DataNumbering'])
+        dataMenu.addAction(self.allActions['DataRegenRefs'])
         dataMenu.addSeparator()
         dataMenu.addAction(self.allActions['DataCloneMatches'])
         dataMenu.addAction(self.allActions['DataDetachClones'])
@@ -578,11 +583,13 @@ class TreeWindow(QMainWindow):
         toolsMenu.addAction(self.allActions['ToolsShortcuts'])
         toolsMenu.addAction(self.allActions['ToolsToolbars'])
         toolsMenu.addAction(self.allActions['ToolsFonts'])
+        toolsMenu.addAction(self.allActions['ToolsColors'])
 
         formatMenu = self.menuBar().addMenu(_('Fo&rmat'))
         formatMenu.addAction(self.allActions['FormatBoldFont'])
         formatMenu.addAction(self.allActions['FormatItalicFont'])
         formatMenu.addAction(self.allActions['FormatUnderlineFont'])
+        formatMenu.addAction(self.allActions['FormatStrikethroughFont'])
         formatMenu.addSeparator()
         # add action's parent to get the sub-menu
         formatMenu.addMenu(self.allActions['FormatFontSize'].parent())
@@ -592,6 +599,7 @@ class TreeWindow(QMainWindow):
         formatMenu.addSeparator()
         formatMenu.addAction(self.allActions['FormatExtLink'])
         formatMenu.addAction(self.allActions['FormatIntLink'])
+        formatMenu.addAction(self.allActions['FormatInsertDate'])
         formatMenu.addSeparator()
         formatMenu.addAction(self.allActions['FormatSelectAll'])
         formatMenu.addAction(self.allActions['FormatClearFormat'])
@@ -741,10 +749,16 @@ class TreeWindow(QMainWindow):
     def saveWindowGeom(self):
         """Save window geometry parameters to history options.
         """
-        globalref.histOptions.changeValue('WindowXSize', self.width())
-        globalref.histOptions.changeValue('WindowYSize', self.height())
-        globalref.histOptions.changeValue('WindowXPos', self.geometry().x())
-        globalref.histOptions.changeValue('WindowYPos', self.geometry().y())
+        contentsRect = self.geometry()
+        frameRect = self.frameGeometry()
+        globalref.histOptions.changeValue('WindowXSize', contentsRect.width())
+        globalref.histOptions.changeValue('WindowYSize', contentsRect.height())
+        globalref.histOptions.changeValue('WindowXPos', contentsRect.x())
+        globalref.histOptions.changeValue('WindowYPos', contentsRect.y())
+        globalref.histOptions.changeValue('WindowTopMargin',
+                                          contentsRect.y() - frameRect.y())
+        globalref.histOptions.changeValue('WindowOtherMargin',
+                                          contentsRect.x() - frameRect.x())
         try:
             upperWidth, lowerWidth = self.breadcrumbSplitter.sizes()
             crumbPercent = int(100 * upperWidth / (upperWidth + lowerWidth))
@@ -787,15 +801,18 @@ class TreeWindow(QMainWindow):
         else:
             if offset:
                 rect.adjust(offset, offset, offset, offset)
-            desktop = QApplication.desktop()
-            if desktop.isVirtualDesktop():
-                # availRect = desktop.screen().rect() # buggy in windows?
-                availRect = (QGuiApplication.screens()[0].
-                             availableVirtualGeometry())
-            else:
-                availRect = desktop.availableGeometry(desktop.primaryScreen())
-            rect = rect.intersected(availRect)
-            self.setGeometry(rect)
+            availRect = QApplication.primaryScreen().availableVirtualGeometry()
+            topMargin = globalref.histOptions['WindowTopMargin']
+            otherMargin = globalref.histOptions['WindowOtherMargin']
+            # remove frame space from available rect
+            availRect.adjust(otherMargin, topMargin,
+                             -otherMargin, -otherMargin)
+            finalRect = rect.intersected(availRect)
+            if finalRect.isEmpty():
+                rect.moveTo(0, 0)
+                finalRect = rect.intersected(availRect)
+            if finalRect.isValid():
+                self.setGeometry(finalRect)
         crumbWidth = int(self.breadcrumbSplitter.width() / 100 *
                          globalref.histOptions['CrumbSplitPercent'])
         self.breadcrumbSplitter.setSizes([crumbWidth,
@@ -876,6 +893,9 @@ class TreeWindow(QMainWindow):
         if (event.type() == QEvent.ActivationChange and
             QApplication.activeWindow() == self):
             self.winActivated.emit(self)
+        elif (event.type() == QEvent.WindowStateChange and
+              globalref.genOptions['MinToSysTray'] and self.isMinimized()):
+            self.winMinimized.emit()
 
     def closeEvent(self, event):
         """Signal that the view is closing and close if the flag allows it.
